<?php

namespace App\Dto\Input;

use ApiPlatform\Metadata\ApiProperty;
use App\Dto\Output\ClientOutput;
use App\Dto\Output\SoftwareProfileOutput;
use App\Entity\Image;
use Symfony\Component\Serializer\Annotation\Groups;
use Symfony\Component\Validator\Constraints as Assert;

final class ImageInput
{
    #[Assert\NotBlank(message: 'validators.image.name.not_blank')]
    #[Groups(['image:write'])]
    #[ApiProperty(description: 'The name of the image', example: "Image 1")]
    public ?string $name = null;

    #[Groups(['image:write'])]
    #[ApiProperty(description: 'The description of the image', example: "Image 1 description")]
    public ?string $description = null;

    #[Groups(['image:write'])]
    #[ApiProperty(description: 'The comments of the image', example: "Image 1 comments")]
    public ?string $comments = null;

    #[Groups(['image:write'])]
    #[ApiProperty(description: 'The type of the image', example: "Server")]
    public ?string $type = null;

    #[Groups(['image:write'])]
    #[ApiProperty(description: 'The path of the image', example: "/path/to/image")]
    public ?string $path = null;

    #[Groups(['image:write'])]
    #[ApiProperty(description: 'The revision of the image', example: "1.0")]
    public ?string $revision = null;

    #[Groups(['image:write'])]
    #[ApiProperty(description: 'The info of the image', example: "Image 1 info")]
    public ?string $info = null;

    #[Groups(['image:write'])]
    #[ApiProperty(description: 'The size of the image', example: 1024)]
    public ?int $size = null;

    #[Groups(['image:write'])]
    #[ApiProperty(description: 'The client of the image')]
    public ?ClientOutput $client = null;

    #[Groups(['image:write'])]
    #[ApiProperty(description: 'The software profile of the image')]
    public ?SoftwareProfileOutput $softwareProfile = null;

    public function __construct(?Image $image = null)
    {
        if (!$image) {
            return;
        }

        $this->name = $image->getName();
        $this->description = $image->getDescription();
        $this->comments = $image->getComments();
        $this->type = $image->getType();
        $this->path = $image->getPath();
        $this->revision = $image->getRevision();
        $this->info = $image->getInfo();
        $this->size = $image->getSize();
        $this->client = new ClientOutput($image->getClient());
        $this->softwareProfile = new SoftwareProfileOutput($image->getSoftwareProfile());
    }

    public function createOrUpdateEntity(?Image $image = null): Image
    {
        if (!$image) {
            $image = new Image();
        }

        $image->setName($this->name);
        $image->setDescription($this->description);
        $image->setComments($this->comments);
        $image->setType($this->type);
        $image->setPath($this->path);
        $image->setRevision($this->revision);
        $image->setInfo($this->info);
        $image->setSize($this->size);
        $image->setClient($this->client->getEntity());
        $image->setSoftwareProfile($this->softwareProfile->getEntity());

        return $image;
    }
}